package org.rcisoft.config;

import org.rcisoft.core.bean.JwtBean;
import org.rcisoft.core.security.JwtAuthenTokenFilter;
import org.rcisoft.core.security.RestAuthenticationEntryPoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.NoOpPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
/**
 * Created by lcy on 17/11/23.
 */
@Configuration
@EnableWebSecurity   //开启WebSecurity支持
@EnableGlobalMethodSecurity(prePostEnabled = true) //开启prePostEnabled注解支持    支持方法级别的安全设置 hasRole()
public class SecurityConfig extends WebSecurityConfigurerAdapter {


    /**
     * 前置过滤器
     * @return
     */
    @Autowired
    JwtAuthenTokenFilter jwtAuthenTokenFilter;


    @Autowired
    private UserDetailsService jwtUserDetailServiceImpl;

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth
                .userDetailsService(jwtUserDetailServiceImpl)
                .passwordEncoder(passwordEncoder());
    }
    /**
     * 密码加密的bean，使用BCrypt
     * @return
     */
    @Bean
    public PasswordEncoder passwordEncoder() {

        //return NoOpPasswordEncoder.getInstance();  明文编码器,spring提供  明文测试用的
        return new BCryptPasswordEncoder();
    }

    /**
     * json web token
     * @return
     */
    @Bean
    JwtBean jwtBean(){
        return new JwtBean();
    }




    @Override
    protected void configure(HttpSecurity http) throws Exception {

        http
                .csrf()     //csrf 对 post delete put 等 进行校验 token 和jwt一样  beforeSecond:{ request.setRequestHeader()}
                .disable()  //由于使用的是JWT，我们这里不需要csrf
                .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)  //禁用session
                .and()
                .authorizeRequests()  //所有请求都要验证
                .antMatchers("/static/**").permitAll()  //过滤 允许
                .antMatchers("/webjars/**").permitAll()
                .antMatchers("/v2/**").permitAll()
                .antMatchers("/swagger-resources/**").permitAll()
                .antMatchers("/swagger-resources/**").permitAll()
                .antMatchers("/api-docs/**").permitAll()
                .antMatchers("/auth/**").permitAll()  //登录注册等请求过滤
                .antMatchers("/code/**").permitAll()  //登录注册等请求过滤
                .antMatchers("/excelUtil/**").permitAll()//excel类
                .antMatchers("/**/**WithoutAuth").permitAll()
                .antMatchers(
                        "/",
                        "/*.html",
                        "/favicon.ico",
                        "/**/*.html",
                        "/**/*.js",
                        "/**/*.css"
                ).permitAll()  //静态资源过滤
                .anyRequest().fullyAuthenticated()
                .and()
                .exceptionHandling()  //验证不通过的配置
                .authenticationEntryPoint(new RestAuthenticationEntryPoint())
        ;
        http   //添加前置过滤器
                .addFilterBefore(jwtAuthenTokenFilter, UsernamePasswordAuthenticationFilter.class);
        http   //禁用header缓存
                .headers().cacheControl();
        /***
         *
         * 验证流程
         *      1.  request-> jwtAuthenTokenFilter  通过jwt 获取token 拼接 UserDetail
         *
         *          jwtAuthenTokenFilter 将 UsernamePasswordAuthenticationToken 放置 securityContext
         *
         *              SecurityContextHolder.getContext().setAuthentication(authentication);
         *
         *      2.  jwtAuthenTokenFilter -> spring security
         *
         *          排除 permitAll() 等验证
         *
         *          验证失败后 authenticationEntryPoint(new RestAuthenticationEntryPoint())
         *
         *
         *
         *    未知  addFilterBefore  addFilterAfter 顺序问题 未确定原因
         *
         */
    }

}
